function pareto = moead_up( mop, vars)
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% -----------
% References:
% -----------
%  Mohammadi, A., Omidvar, M. and Li, X. (2012), "Reference Point Based Multi-objective Optimization Through Decomposition", 
%  in Proceedings of Congress of Evolutionary Computation (CEC 2012), IEEE,  p.1150 - 1157.
% --------
% License:
% --------
% This program is to be used under the terms of the GNU General Public License 
% (http://www.gnu.org/copyleft/gpl.html).
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% Copyright notice: (c) 2014 Asad Mohammadi
% MOEAD_UP runs R-MEAD algorithms for the given mop.
% In this function the base weight vector is calculated based on the
% best weight vector (the weight vector which is associated to the closest solution to the reference point)
% and direction. Then pass the base weight vectors to 
% Upated_weights function to update weights based on the calculated directions.
% The mop must to be minimizing.
% The parameters of the algorithms can be set through varargin which are
% including: 
%   popsize: The subproblem's size.
%   niche: the neighboursize, must less then the popsize.
%   iteration: the total iteration of the moead algorithms before finish.
%   method: the decomposition method, the value can be 'ws' or 'ts'.
    
    starttime = clock;
    %global variable definition.
    global params idealpoint objDim parDim itrCounter;
    %set the random generator.
    rand('state',10);
    
    %Set the algorithms parameters.
    
    paramIn = vars;
    
    density = vars.density;
    radius = vars.radius;
    %ref = vars.ref;
    niche = vars.niche;
    
    %oldWeight= vars.base_weight;
    midIndex = vars.density/2;

    [objDim, parDim, idealpoint, params, subproblems]=init(mop, paramIn);
    
    itrCounter=1;
    while ~terminate(itrCounter)
        tic;
        subproblems = evolve(subproblems, mop, params);
        disp(sprintf('iteration %u finished, time used: %u', itrCounter, toc));
        
        pareto=[subproblems.curpoint];
        pp = [pareto.objective];
        weights = [subproblems.weight];
       
        refs =  vars.ref * ones(1, length(subproblems));
        dist = sqrt(sum((pp-refs).^2));
        
        [Y I] = sort(dist, 'descend');
        [Y bestIndex] = min(dist);
        [Y worstIndex] = max(dist);
        % Direction is calculated based on weight vectors corresponding  to the closest and furthest solutions to the reference point
        dir = subproblems(bestIndex).weight - subproblems(worstIndex).weight;
        bestWeight = subproblems(bestIndex).weight;
        bestWeight = bestWeight / norm(bestWeight, 1);
        
       
       

        base_weight = bestWeight + dir;
        
        subproblems = update_weights(bestWeight', subproblems, density, radius);  
        
        itrCounter=itrCounter+1;
        if (vars.plotting == 1)
            pareto=[subproblems.curpoint];
            pp=[pareto.objective];
            pp = [pp vars.ref];
            figure(1);
            subplot(gca);
            clf;
            hold on;
            plotmop(vars.test, mop);

            if(mop.od == 2)
                scatter(pp(1,:), pp(2,:));
            else
                scatter3(pp(1,:), pp(2,:), pp(3,:));
                view(45, 45);
                view(53, 20);
                box on;                
            end
            drawnow;
            pause(vars.refresh);
        end
        
        pareto=[subproblems.curpoint];
        pp=[pareto.objective];
       
        
    end
    

    pareto=[subproblems.curpoint];
    pp=[pareto.objective];
    
    disp(sprintf('total time used %u', etime(clock, starttime)));
end

function [objDim, parDim, idealp, params, subproblems]=init(mop, vars)
%Set up the initial setting for the MOEA/D.
    objDim=mop.od;
    parDim=mop.pd;    
    idealp=ones(objDim,1)*inf;
    
    %the default values for the parameters.
    params.popsize=vars.popsize;params.niche=vars.niche;params.iteration=vars.iteration;
    params.dmethod=vars.method;
    params.F = 0.5;
    params.CR = 0.5;
    

    subproblems = update_weights(vars.base_weight', [], vars.density, 0.01);
    params.popsize = length(subproblems);
    
    %initial the subproblem's initital state.
    inds = randompoint(mop, params.popsize);
    [V, INDS] = arrayfun(@evaluate, repmat(mop, size(inds)), inds, 'UniformOutput', 0);
    v = cell2mat(V);
    idealp = min(idealp, min(v,[],2));
    
    %indcells = mat2cell(INDS, 1, ones(1,params.popsize));
    [subproblems.curpoint] = INDS{:};
    clear inds INDS V indcells;
 end
    
function subproblems = evolve(subproblems, mop, params, method)
    global idealpoint;
   
    for i=1:length(subproblems)
        %new point generation using genetic operations, and evaluate it.
        ind = genetic_op(subproblems, i, mop.domain, params);
        [obj,ind] = evaluate(mop, ind);
        %update the idealpoint.
        idealpoint = min(idealpoint, obj);
        
        %update the neighbours.
        neighbourindex = subproblems(i).neighbour;
        subproblems(neighbourindex)=update(subproblems(neighbourindex),ind, idealpoint, params.dmethod);
        
        %clear ind obj neighbourindex neighbours;        

        clear ind obj neighbourindex;
    end
end

function subp =update(subp, ind, idealpoint, method)
    if (strcmp(method, 'te') == 1)
        newobj=subobjective([subp.weight], ind.objective,  idealpoint, 'te');
        oops = [subp.curpoint];
        oldobj=subobjective([subp.weight], [oops.objective], idealpoint, 'te' );
        C = newobj < oldobj;
        [subp(C).curpoint]= deal(ind);
        clear C newobj oops oldobj;
    elseif (strcmp(method, 'ws') == 1)
        newobj=subobjective([subp.weight], ind.objective);
        oops = [subp.curpoint];
        oldobj=subobjective([subp.weight], [oops.objective]);
        C = newobj < oldobj;
        [subp(C).curpoint]= deal(ind);
        clear C newobj oops oldobj;
    else 
        warning('unsupported decomposition method');
    end
end
    


function y =terminate(itrcounter)
    global params;
    y = itrcounter>params.iteration;
end
